#-------------------------------------------------------------------------------------------------
# Copyright (c) International Business Machines Corp., 2009
#
# IBM XIV Storage System Software
#
# Licensed materials provided under the terms of the IBM International Program  
# License Agreement. See the Software
# licensing materials that came with the IBM XIV Storage System for terms and
# conditions.
# 
#-------------------------------------------------------------------------------------------------   
import sys

from zope.interface import implements

from twisted.internet import reactor, defer
from twisted.web import server, resource, guard
from twisted.web.woven.simpleguard import MarkAuthenticatedResource
from twisted.python import failure
from twisted.cred import error, credentials
from twisted.cred.portal import IRealm, Portal
from twisted.cred.checkers import ICredentialsChecker

from .config import config
from mpath_management.utils import log

http_log = log.Logger("http")

class LeafMarkAuthenticatedResource(MarkAuthenticatedResource):
    isLeaf = True

class SimpleRealm(object):
    implements(IRealm)

    def __init__(self, resource):
        self.resource = resource

    def requestAvatar(self, avatar_id, mind, *interfaces):
        if resource.IResource in interfaces:
            res = LeafMarkAuthenticatedResource(self.resource, avatar_id)
            return resource.IResource, res, lambda: None
        raise NotImplementedError

class ModifiedLoggingSite(server.Site):
    def __init__(self, *args, **kwargs):
        server.Site.__init__(self, logPath="<dummy>", *args, **kwargs)
        
    def _openLogFile(self, path):
        class LogAdapter(object):
            def write(self, data):
                http_log.debug(data)
            def close(self):
                pass
            
        return LogAdapter()

class AgentUserNamePasswordChecker:
    implements(ICredentialsChecker)

    credentialInterfaces = (credentials.IUsernamePassword,
                            credentials.IUsernameHashedPassword)

    def _cbPasswordMatch(self, matched, username):
        if matched:
            return username
        else:
            return failure.Failure(error.UnauthorizedLogin())

    def requestAvatarId(self, credentials):
        if credentials.username in [ 'manager', 'monitor' ]:
            password = getattr(config, "%s_password" % credentials.username)
            return defer.maybeDeferred(
                credentials.checkPassword, password).addCallback(
                self._cbPasswordMatch, str(credentials.username))
        else:
            return defer.fail(error.UnauthorizedLogin())

    
def run_server(resource, **kwargs):
    if config.require_authentication:
        checker = AgentUserNamePasswordChecker()
        resource = guard.HTTPAuthSessionWrapper(Portal(SimpleRealm(resource),
                                                       [checker]),
                    [ guard.DigestCredentialFactory('md5', 'xiv_mpio_agent') ])

    reactor.listenTCP(port=config.listen_port, interface=config.listen_address,
                      factory=ModifiedLoggingSite(resource))
    reactor.run(**kwargs)
