#-------------------------------------------------------------------------------------------------
# Copyright (c) International Business Machines Corp., 2009
#
# IBM XIV Storage System Software
#
# Licensed materials provided under the terms of the IBM International Program  
# License Agreement. See the Software
# licensing materials that came with the IBM XIV Storage System for terms and
# conditions.
# 
#-------------------------------------------------------------------------------------------------   
import _winreg as winreg
from _winreg import HKEY_LOCAL_MACHINE as HKLM
from _winreg import OpenKey, CreateKey, SetValueEx, QueryValueEx, REG_DWORD, REG_BINARY, REG_SZ, REG_NONE, KEY_READ, KEY_WRITE

SERVICE_NAME = 'XIVmpioAgent'
SERVICE_PARAMETERS_KEY = 'SYSTEM\CurrentControlSet\Services\%s\Parameters' % SERVICE_NAME

class WindowsConfigRegistry(object):
    def __init__(self):
        self._init_registry_objects()

    def load_value(self, key_name, default_value):
        try:
            val = QueryValueEx(self.registry_key_object, key_name)[0]
            if isinstance(default_value, bool): # hack to know expected type.
                val = val != 0
            if isinstance(default_value, float):
                val = float(val)
            return val
        except:
            return default_value

    def save_value(self, key_name, value):
        try:    
            registry_key = OpenKey(HKLM, self.registry_key_path, 0, KEY_WRITE)
            SetValueEx(registry_key, key_name, 0, self._type_to_reg_type(value),
                       value)
        except (WindowsError, 2):
            pass

    def _init_registry_objects(self):
        self.registry_key_path = SERVICE_PARAMETERS_KEY
        self.registry_key_object = self._get_or_create_registry_key_object()
       
    def _get_or_create_registry_key_object(self):
        registry_key = None
        try:    
            registry_key = OpenKey(HKLM, self.registry_key_path, 0, KEY_READ)
        except (WindowsError, 2):
            registry_key = CreateKey(HKLM, self.registry_key_path)
        return registry_key

    def _type_to_reg_type(self, value):
        if value is None:
            return REG_NONE
        if isinstance(value, int):
            return REG_DWORD
        if isinstance(value, str):
            return REG_SZ
        if isinstance(value, float):
            return REG_SZ
